/*
 * JCuda - Java bindings for NVIDIA CUDA driver and runtime API
 * http://www.jcuda.org
 *
 * Copyright 2009-2012 Marco Hutter - http://www.jcuda.org
 */

package org.jcuda.samples.matrix.cg;

import jcuda.Pointer;

/**
 * Interface for a CG solver that operates on matrices in CSR
 * format that are stored on the device
 */
interface CGSolverDevice extends CGSolver, DeviceData
{
    /**
     * Prepare this solver to solve an equation involving the
     * given matrix. Any data structures that may already have
     * been allocated by a previous call to this method will
     * be released by calling the {@link #dispose()} method,
     * if necessary.
     * 
     * @param matrix The matrix
     */
    void setup(FloatMatrixDeviceCSR matrix);
    
    /**
     * Solve the equation <code>matrix * x = b</code> for <code>x</code>,
     * using the matrix that was previously passed to the 
     * {@link #setup(FloatMatrixDeviceCSR)} method.
     * 
     * @param x The solution
     * @param b The right hand side of the equation
     */
    void solve(Pointer x, Pointer b);
}