/*
 * JCuda - Java bindings for NVIDIA CUDA driver and runtime API
 * http://www.jcuda.org
 *
 * Copyright 2009-2012 Marco Hutter - http://www.jcuda.org
 */

package org.jcuda.samples.matrix.cg;

import static jcuda.runtime.JCuda.cudaFree;
import jcuda.driver.CUdeviceptr;

/**
 * Default implementation of a {@link FloatMatrixDeviceDense}.
 * The data is stored in column-major order
 */
class DefaultFloatMatrixDeviceDense implements FloatMatrixDeviceDense
{
    /**
     * The number of rows 
     */
    private final int numRows;
    
    /**
     * The number of columns 
     */
    private final int numCols;

    /**
     * The data 
     */
    private final CUdeviceptr data;
    
    /**
     * Creates a new matrix with the given number of rows and columns,
     * and the given data
     * 
     * @param numRows The number of rows
     * @param numCols The number of columns
     * @param data The data 
     */
    DefaultFloatMatrixDeviceDense(int numRows, int numCols, CUdeviceptr data)
    {
        this.numRows = numRows;
        this.numCols = numCols;
        this.data = data;
    }
    
    @Override
    public void dispose()
    {
        cudaFree(data);
    }
    
    @Override
    public int getNumRows()
    {
        return numRows;
    }
    @Override
    public int getNumCols()
    {
        return numCols;
    }
    @Override
    public CUdeviceptr getData()
    {
        return data;
    }
    
    @Override
    public String toString()
    {
        StringBuilder sb = new StringBuilder();
        sb.append("Copied from device:\n");
        sb.append(Matrices.createHostDense(this));
        return sb.toString();
    }

    
}